/*
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2016 KYOCERA Corporation
 *
 * Present statistics on encrypted eCryptfs file attributes
 */

#define LOG_TAG "ecryptfs_check"

#include <dirent.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <errno.h>
#include <stdbool.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include "../include/ecryptfs.h"

#include <cutils/log.h>
#include <cutils/properties.h>

int do_check_files(DIR *d);
int check_file(int fd);

bool encrypt_file_check = true;

static void usage(const char *filename)
{
    printf("Usage:\n\n"
           "%s <filename>\n", filename);
}

int main(int argc, const char *argv[])
{
    int rc = -1;
    if (argc == 3) {
        if (!strcmp(argv[2], "plain")) {
            encrypt_file_check = false;
        } else if (!strcmp(argv[2], "encrypt")) {
            encrypt_file_check = true;
        }
    }

    printf("encrypt_file_check %d\n",encrypt_file_check);
    __android_log_print(ANDROID_LOG_INFO,LOG_TAG,"encrypt_file_check %d\n",encrypt_file_check);

    DIR *d = opendir(argv[1]);
    if(d) {
        rc = do_check_files(d);
        closedir(d);

        printf("rc %d\n",rc);
        __android_log_print(ANDROID_LOG_INFO,LOG_TAG,"rc %d\n",rc);
    } else {
        printf("Argument error. Please set directory.");
        __android_log_print(ANDROID_LOG_INFO,LOG_TAG,"Argument error. Please set directory.");

    }

    if (rc) {
        property_set("vold.ecryptfs_check","-1");
        __android_log_print(ANDROID_LOG_INFO,LOG_TAG,"check ret -1");
        return -1;
    } else {

        property_set("vold.ecryptfs_check","1");
        __android_log_print(ANDROID_LOG_INFO,LOG_TAG,"check ret 1");
        return 0;
    }
}

int do_check_files(DIR *d)
{
    struct dirent *de;
    int dfd, fd;
    DIR *d2;
    int rc = 0;

    dfd = dirfd(d);
    while((de = readdir(d))) {
        if (de->d_name[0] == '.')
            continue;

        if (de->d_type == DT_DIR)
        {
            printf("dir %s\n",de->d_name);
            fd = openat(dfd, de->d_name, O_RDONLY | O_DIRECTORY);
            if(fd < 0)
                continue;

            d2 = fdopendir(fd);
            if(d2 == 0)
                close(fd);
            else {
                rc = do_check_files(d2);
                closedir(d2);
                if (rc != 0) {
                    return rc;
                }
            }
        } else if (de->d_type == DT_REG) {
            fd = openat(dfd, de->d_name, O_RDONLY);
            rc = check_file(fd);
            printf("%s  %d\n",de->d_name,rc);
            if (rc != 0) {
                return rc;
            }
        }
    }
    return rc;
}

int check_file(int fd)
{
    ssize_t quant_read;
    struct ecryptfs_crypt_stat_user crypt_stat;
    char buf[4096];
    int rc = -1;

    if (fd == -1) {
        printf("Error opening file for RD_ONLY access; errno msg = [%m]\n");
        __android_log_print(ANDROID_LOG_INFO,LOG_TAG,"Error opening file for RD_ONLY access; errno msg = [%m]\n");
        rc = -EIO;
        goto out;
    }
    quant_read = read(fd, buf, 4096);
    if (quant_read == -1) {
        printf("Error attempting to read from file; errno msg = [%m]\n");
        __android_log_print(ANDROID_LOG_INFO,LOG_TAG,"Error attempting to read from file; errno msg = [%m]\n");
        rc = -EIO;
        goto out;
    }
    rc = ecryptfs_parse_stat(&crypt_stat, buf, quant_read);
    if (rc) {
        printf("Valid eCryptfs metadata information not found\n");
        __android_log_print(ANDROID_LOG_INFO,LOG_TAG,"Valid eCryptfs metadata information not found\n");

        if (!encrypt_file_check) {
            rc = 0;
        }
        goto out;
    }
    if (crypt_stat.flags & ECRYPTFS_ENCRYPTED) {
        if (encrypt_file_check) {
            rc = 0;
        } else {
            rc = -1;
        }
    }

out:
    if (fd != -1)
        close(fd);

    return rc;
}
